<?php if ( ! defined('BASEPATH')) exit('No direct script access allowed');

require_once(APPPATH . '/models/invoices/mod.invoices_pmf.php');

class Invoices_admin extends Invoices_pmf
{
	public	$options	= array();

	function __construct()
	{
		parent::__construct();
	}
	
	public function get_list($select = '', $options = array())
	{
		if (!empty($options['filters']['inv.is_deposit']) && empty($options['status_count']))
			$select = "inv.invoice_id, inv.invoice_number, inv.estimate_id, DATE_FORMAT(inv.invoice_date, '%Y-%m-%d') as invoice_date, DATE_FORMAT(inv.due_date, '%Y-%m-%d') as due_date, inv.status, inv.amount, inv.currency_code, c.client_id, concat(c.client_code, c.client_name) AS client_name,inv.is_deposit, inv.payment_amount, inv.deposit_balance, inv.is_archive, DATE_FORMAT(inv.last_updated_date, '%Y-%m-%d') as last_updated_date";
		$query = $this->invoice_list_query($select, $options);
		
		return $query->result_array();
	}
	
	public function get_status_count($select = '', $options = array())
	{
		if (!$select) $select = 'count(inv.invoice_id) AS count, case when inv.is_archive=1 then "a" else status end as status_res';
		return $this->get_list($select, $options);
	}	
	
	
	public function save($data)
	{
		$this->load->library('invoice_table');
		
		$params = array();
		if (!empty($data[$this->invoice_table->tbl_key]))
		{
			$params['where'] = array('entity_id' => $this->ot_app->get('entity_id'), 'invoice_id' => $data[$this->invoice_table->tbl_key]);
		}
		$result = $this->invoice_table->save($data,$params);
		 
		return $result;
	}
	
	public function update_status ($data)
	{
		$data['last_updated_by']	= $this->ot_user->get('user_id');
		$data['last_updated_date']	= $this->auth->get_timezone_date('','UTC');
		$send_log=0;
		if (!empty($data['send_log'])) {
			$send_log = $data['send_log'];
			unset($data['send_log']);
		}
		
		$result	= $this->get_list("inv.invoice_id, inv.invoice_number, inv.amount, inv.currency_code", array('filters' => array('inv.invoice_id'=>$data['invoice_id'])));
		$client = $this->client_contacts($data['invoice_id'], 'cc.name');
		$this->db->where(array('entity_id' => $this->ot_app->get('entity_id'), 'invoice_id' => $data['invoice_id'],'is_archive' => 0));
		unset($data['invoice_id']);
		$this->db->update('invoice', $data);
		if (!$this->db->affected_rows()) {
			return false;
		}
		// Insert invoice activity
		$CI =& get_instance();
		$result = $result[0];
		$fullname = $CI->ot_user->get('first_name').' '.$CI->ot_user->get('last_name');
		$currency_symbol = get_currency($result['currency_code']);
		$act_data = array(
				'invoice_id' 	=> $result['invoice_id'],
				'a_mode'		=> 'IS',
				'a_notes'		=>  $fullname. ' sent Invoice #'.$result['invoice_number'].' ('.$currency_symbol.' '.$result['amount'].') to '.$client[0]['name']
		);			
		if ($send_log && ! invoice_activity($act_data))
				return false;
		
		$act_data['a_notes'] = $fullname. ' updated status of Invoice #'.$result['invoice_number'].' ('.$currency_symbol.' '.$result['amount'].') as sent';
		if (!$send_log && ! invoice_activity($act_data))
				return false;
		
		return $this->db->affected_rows();
	}
	
	public function client_contacts($invoice_id, $select = 'concat(cc.name, "<",a.email,">") AS email, inv.client_contact_id as inv_contact_id, cc.client_contact_id')
	{	
		$this->db->select($select, false);
		$this->db->from('invoice AS inv');
		$this->db->join('client_contact AS cc', 'cc.client_id = inv.client_id');
		$this->db->join('address AS a', 'a.address_id = cc.address_id');
		
		$this->db->where(array('inv.entity_id' => $this->ot_app->get('entity_id'), 'inv.invoice_id' => $invoice_id));
		$this->db->order_by('inv.invoice_id');
		
		
		$query	= $this->db->get();
		$result =  $query->result_array();
		
		return $result; 
	}
	
	public function outstanding_amount($data)
	{
		$this->db->select('sum(inv.amount - inv.payment_amount) AS outstanding_amount');
		$this->db->from('invoice AS inv');
		$this->db->where(array('inv.entity_id' => $this->ot_app->get('entity_id'), 'inv.invoice_id' => $data['invoice_id'], 'inv.client_id' => $data['client_id']));
		
		$query	= $this->db->get();
		$row =  $query->row_array();	

		return $row;
	}
	
	public function get_payment_details($invoice_id)
	{
		$this->db->select('invp.*');
		$this->db->from('invoice_payment AS invp');
		$this->db->where('invp.invoice_id', $invoice_id);
		$query	= $this->db->get();

		return $query->result_array();
	}
	
	public function payment_save($data)
	{
		$this->load->library('invoice_table');
		$result = array();

		// payment via deposit invoice
		if (!empty($data['deposit_payment']))
		{
			$this->db->select('inv.invoice_id, inv.deposit_balance, inv.currency_code, inv.invoice_number');
			$this->db->from('invoice AS inv');
			$where = array('inv.entity_id' => $this->ot_app->get('entity_id'), 'inv.client_id' => $data['client_id'], 'inv.is_deposit' => 1, 'inv.currency_code' => $data['currency_code'],'inv.deposit_balance !=' => 0);
			$this->db->where($where);
			$this->db->order_by('inv.invoice_id');
			$query = $this->db->get();
			$deposits = $query->result_array();

			if (!$deposits)
			{
				return false;
			}
			
			foreach ($deposits as $row)
			{
				unset($data['deposit_payment']);unset($data['currency_code']);unset($data['client_id']);
				$prepare_data = $data;
				$prepare_data['deposit_invoice_id'] = $row['invoice_id'];
				$prepare_data['payment_number'] = $row['invoice_number'];
				if ($data['amount']>=$row['deposit_balance'])
				{
					$prepare_data['amount'] = $row['deposit_balance'];
					$data['amount']-= $row['deposit_balance'];
					$row['deposit_balance'] = 0;
				} else {
					$row['deposit_balance']-=$data['amount'];
					$data['amount'] = 0;
				}
				
				if ($return = $this->invoice_table->payment_save($prepare_data))
				{
					// update deposit invoice
					$this->db->where(array('invoice_id' => $row['invoice_id']));
					$this->db->update('invoice', array('deposit_balance' => $row['deposit_balance']));
				}
				if ($data['amount']==0) break;
			}
			if ($return)
			{
					$result= $this->response->prepare($this->get_payment_details($data['invoice_id']));
					$result['status'] = $return['status'];
					
			}
		} else {
			$result =  $this->invoice_table->payment_save($data);
		}
		return $result;
	}
	
	public function outstanding_credit($client_id, $currency_code='')
	{
		// get outstanding credit
		$this->db->select('sum(inv.deposit_balance) AS deposit_balance, inv.currency_code');
		$this->db->from('invoice AS inv');
		$where = array('inv.entity_id' => $this->ot_app->get('entity_id'), 'inv.client_id' => $client_id, 'inv.is_deposit' => 1);
		if ($currency_code)
		{
			$where['inv.currency_code'] = $currency_code; 
		}
		$this->db->where($where);
		$this->db->group_by('inv.client_id');
		
		$query		= $this->db->get();
		return $query->row_array();
	}
	
	public function get_invoice_applied($invoice_id)
	{
		$this->db->select('inv.invoice_id, inv.currency_code, sum(invp.amount) AS amount, inv.invoice_number');
		$this->db->from('invoice AS inv');
		$this->db->join('invoice_payment AS invp', 'inv.entity_id=c.entity_id and inv.invoice_id=invp.invoice_id');
		$this->db->where(array('inv.entity_id'=>$this->ot_app->get('entity_id'),'invp.deposit_invoice_id'=>$invoice_id));
		$this->db->group_by('inv.invoice_number, inv.currency_code');
		
		$query = $this->db->get();
		//echo $this->db->last_query();
		$result = $query->result_array(); 
		return $result;
	}

	public function filter_where($invoice_id)
	{
			$where['invoice_id']	= $invoice_id;
			$where['entity_id']		= $this->ot_app->get('entity_id');
			
			return $where;
	}
	
	public function archive_status($invoice_id, $status)
	{
		$this->db->where(array('invoice_id' => $invoice_id));
		$this->db->update('invoice', array('is_archive' => ($status)?1:0));

		return $this->db->affected_rows();
	}
	
	public function payment_delete($pay_id)
	{
		$pay_id	= (int)$pay_id;

		if (!$pay_id)
		{
			return false;
		}

		$query	= $this->db->get_where('invoice_payment', array('invoice_payment_id' => $pay_id));
		$pay_detail	= $query->row_array();

		if ($pay_detail && $this->db->delete('invoice_payment', array('invoice_payment_id' => $pay_id)))
		{
			$this->db->set('payment_amount', 'payment_amount - '.$pay_detail['amount'], false);
			$this->db->set('status', '(case when payment_amount>0 then "b" else "s" end)', false);
			$this->db->where(array('invoice_id' => $pay_detail['invoice_id']));
			$this->db->update('invoice');

			// activity log for invoice delete
			$fullname = $this->ot_user->get('first_name').' '.$this->ot_user->get('last_name');
			// $currency_symbol = get_currency($invoice['currency_code']);
			$act_data = array(
					'invoice_id' 	=> $pay_detail['invoice_id'],					
					'a_mode'		=> 'IP',
					'a_notes'		=>  $fullname. ' deleted payment of '.$pay_detail['amount'].' for Invoice #'.$pay_detail['invoice_id']
			);
			
			invoice_activity($act_data);

			return true;
		}

		return false;
	}
}
