Date.monthNames		= ['January', 'February', 'March', 'April', 'May', 'June', 'July', 'August', 'September', 'October', 'November', 'December'];
Date.shortMonth		= ['Jan', 'Feb', 'Mar', 'Apr', 'May', 'Jun', 'Jul', 'Aug', 'Sep', 'Oct', 'Nov', 'Dec'];
Date.dayNames		= ['Sunday', 'Monday', 'Tuesday', 'Wednesday', 'Thursday', 'Friday', 'Saturday'];
Date.shortDays		= ['Sun', 'Mon', 'Tue', 'Wed', 'Thu', 'Fri', 'Sat'];
Date.tinyDays		= ['S', 'M', 'T', 'W', 'T', 'F', 'S'];
Date.firstDayOfWeek	= 0;
Date.defaultFormat	= "MMM dd,yyyy";

// trimming with array ops
String.prototype.trim	= function(){
	return this.split(/\s/).join(' ');
};

// string replicator 
String.prototype.times	= function(n){
	var s	= '';
	for(var i=0;i<n;i++)
		s	+= this;
	return s;
};

// zero padding and trailing
String.prototype.zp	= function(n){
	return '0'.times(n-this.length) + this;
};
String.prototype.zt	= function(n){
	return this + '0'.times(n-this.length);
};

// string reverse
String.prototype.reverse	= function(){
	return this.split('').reverse().join('');
};

// clear format from a string representation of a number
String.prototype.clean	= function(){
	return parseFloat(this.replace(/[^0-9|.|-]/g,''));
};

Number.prototype.zp	= function(n){
	return this.toString().zp(n);
};

// the date format prototype
Date.prototype.format	= function(f, is24hr) {
	if(!this.valueOf())
		return null;

	var d	= this;

	return f.replace(/(yyyy|mmmm|mmm|mm|dddd|ddd|dd|d|hh|oo|nn|ss|ap|a\/p)/gi,
        function($1) {
        	switch($1.toLowerCase()) {
        		case 'yyyy':
        			return d.getFullYear();
        		case 'mmmm':
        			return Date.monthNames[d.getMonth()];
        		case 'mmm':
        			return Date.shortMonth[d.getMonth()];
        		case 'mm':
        			return (d.getMonth()+1).zp(2);
        		case 'dddd':
        			return Date.dayNames[d.getDay()];
        		case 'ddd':
        			return Date.shortDays[d.getDay()];
        		case 'dd':
        			return d.getDate().zp(2);
        		case 'd':
        			return Date.tinyDays[d.getDay()];
        		case 'hh':
        			if (is24hr)
        				return (h=d.getHours().zp(2));
        			else
        				return ((h=d.getHours()%12)?h:12).zp(2);
        		case 'oo':
        			return d.getHours().zp(2);
        		case 'nn':
        			return d.getMinutes().zp(2);
        		case 'ss':
        			return d.getSeconds().zp(2);
        		case 'a/p':
        		case 'ap':
        			return d.getHours()<12?'AM':'PM';
        	}
        }
    );
};

if (!Date.prototype.getFullYear)
{
	Date.prototype.getFullYear	= function(){
		var yy = this.getYear();
		return (yy < 1900 ? yy + 1900 : yy);
	};
}

Date.prototype.addSeconds	= function(seconds) {
	this.setSeconds(this.getSeconds() + Number(seconds));
	return this;
};

Date.prototype.addMinutes	= function(minutes) {
	this.setMinutes(this.getMinutes() + Number(minutes));
	return this;
};

Date.prototype.addHours	= function(hours) {
	this.setHours(this.getHours() + Number(hours));
	return this;
};

var addLeave	= function(dt){
	if (dt.isLeave())
	{
		dt.addDays(1);
		addLeave(dt);
	}
	return dt;
};
var addLeaveNeg	= function(dt){
	if (dt.isLeave())
	{
		dt.addDays(-1);
		addLeave(dt);
	}
	return dt;
};
Date.prototype.addLeaveDays	= function(days){
	var dt	= new Date(this);
	days	= Number(days);

	if(days > 0)
	{
		for (var i=1; i<=days; i++)
		{
			addLeave(dt).addDays(1);
		}
		addLeave(dt);
	}
	else if(days < 0)
	{
		for (var i=-1; i>=days; i--)
		{
			addLeaveNeg(dt).addDays(-1);
		}
		addLeaveNeg(dt);
	}

	this.setTime(dt.getTime());
	return this;
};

Date.prototype.addDays	= function(days) {
	this.setDate(this.getDate() + Number(days));
	return this;
};

Date.prototype.addWeeks	= function(weeks) {
	weeks	= Number(weeks) || 1;
	this.setDate(this.getDate() + (7*weeks));
	return this;
};

Date.prototype.addMonth	= function(months) {
	this.setMonth(this.getMonth() + Number(months));
	return this;
};

Date.prototype.addYear	= function(years) {
	this.setFullYear(this.getFullYear() + Number(years));
	return this;
};

Date.prototype.getWeek	= function() {
	var onejan	= new Date(this.getFullYear(), 0, 1);
	return Math.ceil((((this - onejan) / 86400000) + onejan.getDay()+1)/7);
};

Date.prototype.isAM	= function(){
	return (this.getHours() < 12);
};

Date.prototype.isPM	= function(){
	return (this.getHours() > 12);
};

Date.prototype.isBefore	= function(date2) {
	if (date2 == null) return false;
	return (this.getTime() < date2.getTime());
};

Date.prototype.isAfter	= function(date2) {
	if (date2 == null) return false;
	return (this.getTime() > date2.getTime());
};

Date.prototype.clearTime	= function() {
	this.setHours(0);
	this.setMinutes(0);
	this.setSeconds(0);
	this.setMilliseconds(0);
	return this;
};

Date.prototype.equals	= function(date2, dateOnly) {
	if (date2 == null) return false;

	if (dateOnly)
	{
		return (new Date(this.getTime()).clearTime().getTime() == new Date(date2.getTime()).clearTime().getTime());
	}

	return (this.getTime() == date2.getTime());
};

Date.prototype.getDayName	= function(short_name) {
	if (short_name)
	{
		return Date.shortDays[this.getDay()];
	}
	return Date.dayNames[this.getDay()];
};

Date.prototype.getMonthName	= function(short_name) {
	if (short_name)
	{
		return Date.shortMonth[this.getMonth()];
	}
	return Date.monthNames[this.getMonth()];
};

Date.prototype.toInt	= function() {
	return this.getFullYear() * 10000 + (this.getMonth() + 1) * 100 + this.getDate();
};

Date.fromInt	= function(dateInt) {
	var year	= parseInt(dateInt / 10000);
	var month	= parseInt((dateInt - year * 10000) / 100);
	var day		= parseInt(dateInt - year * 10000 - month * 100);

	return new Date(year, month - 1, day, 12, 00, 00);
};

Date.prototype.isToday	= function(timeZoneValue) {
	timeZoneValue	= timeZoneValue || Date.timeZoneValue;
	if (timeZoneValue)
	{
		var utc	= new Date().getTime() + (new Date().getTimezoneOffset() * 60000);
		return this.toInt() == new Date(utc + (3600000 * Number(timeZoneValue))).toInt();
	}

	return this.toInt() == new Date().toInt();
};
Date.prototype.isYesterday	= function(timeZoneValue) {
	return new Date(this).addDays(1).isToday(timeZoneValue);
};
Date.prototype.isTomorrow	= function(timeZoneValue) {
	return new Date(this).addDays(-1).isToday(timeZoneValue);
};
Date.prototype.isFuture	= function(timeZoneValue) {
	timeZoneValue	= timeZoneValue || Date.timeZoneValue;
	if (timeZoneValue)
	{
		var utc	= new Date().getTime() + (new Date().getTimezoneOffset() * 60000);
		var cur_dat = new Date(utc + (3600000 * Number(timeZoneValue)));
		cur_dat.setHours(0,0,0,0);
		cur_dat.setDate(cur_dat.getDate() + 1);
		return this.getTime() >= cur_dat.getTime();
	}

	return this.getTime() > new Date().getTime();
};
Date.prototype.isPast	= function(timeZoneValue) {
	timeZoneValue	= timeZoneValue || Date.timeZoneValue;
	if (timeZoneValue)
	{
		var utc	= new Date().getTime() + (new Date().getTimezoneOffset() * 60000);
		var cur_dat = new Date(utc + (3600000 * Number(timeZoneValue)));
		cur_dat.setHours(0,0,0,0);
		return this.getTime() < cur_dat.getTime();
	}

	return this.getTime() < new Date().getTime();
};
Date.prototype.getWeekStart	= function(firstDayOfWeek){
	firstDayOfWeek	= firstDayOfWeek || Date.firstDayOfWeek;
	var ret	= new Date(this);
	return new Date(ret.setDate(ret.getDate() - Math.abs(ret.getDay() - firstDayOfWeek)));
};

Date.prototype.getWeekEnd	= function(firstDayOfWeek){
	firstDayOfWeek	= firstDayOfWeek || Date.firstDayOfWeek;
	return this.getWeekStart(firstDayOfWeek).addDays(6);
};
Date.prototype.isThisWeek	= function(){
	return new Date(this) > new Date().getWeekStart() && new Date(this) < new Date().getWeekEnd();
};
Date.prototype.isLastWeek	= function(){
	var dt	= new Date().addDays(-7);
	return new Date(this) > dt.getWeekStart() && new Date(this) < dt.getWeekEnd();
};
Date.prototype.isNextWeek	= function(){
	var dt	= new Date().addDays(7);
	return new Date(this) > dt.getWeekStart() && new Date(this) < dt.getWeekEnd();
};
Date.prototype.isThisMonth	= function(){
	return new Date(this) > new Date().getMonthStart() && new Date(this) < new Date().getMonthEnd();
};
Date.prototype.isLastMonth	= function(){
	var dt	= new Date().addDays(-7);
	return new Date(this) > dt.getMonthStart() && new Date(this) < dt.getMonthEnd();
};
Date.prototype.isNextMonth	= function(){
	var dt	= new Date().addDays(7);
	return new Date(this) > dt.getMonthStart() && new Date(this) < dt.getMonthEnd();
};
Date.prototype.setFirstDayOfThisWeek	= function(firstDayOfWeek) {
	firstDayOfWeek	= firstDayOfWeek || Date.firstDayOfWeek;
	this.setDate(this.getDate() - this.getDay() + firstDayOfWeek - (this.getDay() == 0 && firstDayOfWeek != 0 ? 7 : 0));
};
Date.prototype.getNthWeek	= function(){
	var onejan	= new Date(this.getFullYear(),0,1);
	return Math.ceil((((this - onejan) / 86400000) + onejan.getDay()+1)/7);
};
Date.prototype.dayPos	= function() {
	return Date.dayNames.indexOf(this.format("dddd").ucfirst())+1;
};

Date.prototype.getMonthStart	= function(setToThis){
	var newDate	= new Date(this.getFullYear(), this.getMonth());
	if (setToThis)
	{
		this.setTime(newDate.getTime());
	}
	return newDate;
};

Date.prototype.getMonthEnd	= function(setToThis){
	var newDate	= new Date(this.getFullYear(), this.getMonth()+1, 0);
	if (setToThis)
	{
		this.setTime(newDate.getTime());
	}
	return newDate;
};

Date.prototype.isLeave	= function() {
	return isLeave(this);
};

function isLeave(date)
{
	if (!Date.hasLeave)
	{
		return false;
	}

	date	= date ? new Date(date) : new Date();

	// Days check
	if (Otime.leave.days.indexOf(date.getDay()) >= 0)
	{
		return true;
	}

	var dts	= Otime.leave.dates;

	// Repeat dates checking.
	if (dts.repeat.indexOf(date.format("MM/DD")) >= 0)
	{
		return true;
	}

	// Non-Repeat Dates checking.
	if (dts.non_repeat.indexOf(date.format(Otime.jsDateFormat)) >= 0)
	{
		return true;
	}

	return false;
}

Date.getTotalLeave	= function(from, to){
	if (!Date.hasLeave)
	{
		return 0;
	}

	from = new Date(from);
	to = new Date(to);
	var totDays	= ((to.getTime() - from.getTime()) / (24*60*60*1000));
	var totLeave = 0;
	for (var i=0; i<=Number(totDays); i++)
	{
		if (from.isLeave())
		{
			totLeave++;
		}
		from.addDays(1);
	}
	return Number(totLeave);
};
Date.checkLeave	= function(dt, less){
	dt	= new Date(dt);
	if (dt.isLeave())
	{
		return Date.checkLeave(dt.addDays(less?-1:1), less);
	}
	return dt;
};
